import 'dart:async';

import 'package:flutter/material.dart';

typedef SnackBarBuilder<T> = Future<SnackBar> Function(
  BuildContext context,
  T config,
);

class ScaffoldMessengerProxy<T> extends StatefulWidget {
  const ScaffoldMessengerProxy({
    required this.child,
    required this.stream,
    required this.snackBarBuilder,
  });

  final Widget child;
  final Stream<T> stream;
  final SnackBarBuilder<T> snackBarBuilder;

  @override
  State<StatefulWidget> createState() => _ScaffoldMessengerProxyState<T>();
}

class _ScaffoldMessengerProxyState<T> extends State<ScaffoldMessengerProxy<T>> {
  final _ScaffoldMessengerProxyInternal<T> _internal =
      _ScaffoldMessengerProxyInternal<T>();

  static late BuildContext _context;

  @override
  void dispose() {
    _internal.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    _context = context;
    _internal.subscription ??= widget.stream.listen(
      (_config) async {
        final SnackBar _snackBar =
            await widget.snackBarBuilder(_context, _config);
        ScaffoldMessenger.of(_context).showSnackBar(_snackBar);
      },
      onDone: () => _internal.dispose(),
    );
    return widget.child;
  }
}

class _ScaffoldMessengerProxyInternal<T> {
  _ScaffoldMessengerProxyInternal();
  StreamSubscription<T>? subscription;

  void dispose() {
    subscription?.cancel();
  }
}
